use std::ops::Deref;

use actix_web::{get, web, Responder, Result};
use serde::Serialize;

use bank::account::{Account, AccountError};

use crate::handlers::error::HttpAccountError;
use crate::AppState;

#[derive(Serialize)]
pub struct JsonAccount {
    pub number: String,
    pub owner: String,
    pub balance: f64,
    pub is_active: bool,
}

impl From<&Account> for JsonAccount {
    fn from(a: &Account) -> Self {
        JsonAccount {
            number: a.number.clone(),
            owner: a.owner.clone(),
            balance: a.balance,
            is_active: a.is_active,
        }
    }
}

#[get("/{nr}")]
pub async fn route(info: web::Path<String>, data: web::Data<AppState>) -> Result<impl Responder> {
    let nr = info.into_inner();
    info!("getting account {}...", nr);

    let bank = data.bank.read().unwrap();
    match bank.accounts.get(&nr) {
        None => Err(HttpAccountError(AccountError::NotFound).into()),
        Some(account) => {
            let account = account.read().unwrap();
            let account: JsonAccount = account.deref().into();
            Ok(web::Json(account))
        }
    }
}
